<?php

/**
 * CompositeBehavior Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This class is an implementation of the composite design-pattern. 
 * In brief: this class enables us to create related objects on-the-fly without making much effort.  
 * 
 * @todo: implement automagic update upon change of the parent
 */
class CompositeBehavior extends CActiveRecordBehavior{
    
    /**
     * @var string  the Type of the new object (e.g.: Aco/Aro)
     */
    protected $type = NULL;
    
    /**
     * @var array   the fields the new object should take from its parent: array('parentKey' => 'newObjectKey') 
     */
    protected $additionalFields = array();
    
    /**
     * Some fields (e.g. autoincremented ID) should not be written to the Database. Better solutions are always welcome ;)
     * @todo remove this dependency in a lightweight way
     * @var array
     */
    protected $fieldsToDiscard = array('id'); 
    
    /**
     * @var int Values to track the save of a new object
     */
    protected $old_id = NULL, $id = NULL;
    
    public function __construct($type, $foreignKey, $additionalFields){
        $this->type = $type;
        $this->additionalFields = $additionalFields;
    }
    
    public function afterFind($evt){
        $this->old_id = $this->id;
    }
    
    /**
     * This method updates all the children of the object on change
     * @param CEvent $evt the event of saving
     * @return boolean 
     */
    public function afterSave($evt){
        //Check the record is new
        $id = $evt->sender->getAttribute('id');
        if($this->old_id != $id){
            //Now add the new object to the table
            $class = $this->type;
            $obj = new $class;

            foreach($this->additionalFields as $key => $field){
                $obj->$field = $evt->sender->getAttribute($key);
            }
            
            if($this->old_id == NULL && $id == NULL){
                foreach($this->fieldsToDiscard as $field){
                    $obj->$field = NULL;
                }
            }
            
            return $obj->save();
        }
    }
    
}
?>
