<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

class Tree{
    
    protected $table = NULL;
    protected $relatedTable = NULL;
    protected $type = NULL;
    
    //relatedTable: normally aco_aro, type: aro or aco
    public function __construct($table, $type, $relatedTable = '{{aco_aro}}'){
        $this->table = $table;
        
        $this->type = $type;
        $this->relatedTable = $relatedTable;
    }
    
    /**
     * For the sake of performance the normal tenets have been broken. 
     * Thereupon lft and rgt-values are available in both {{aco/aro}} and {{aco_aro}}
     * Whenever we do something complex and we don't bother to update the values on-the-fly (the tree only operates on the single table and
     * not on the MANY-TO-MANY-relations) we just updated the hole thing
     * @todo: Keep the eyes peeled for a better solution. Though I doubt there is any. 
     * @todo: remove keys, process update and restore them later 
     */
    public function synchronizeDenormalization(){
        
        $cmd = Yii::app()->db->createCommand(
                'UPDATE `'.$this->table.'` AS single '
                .', `'.$this->relatedTable.'` AS related'
                .' SET related.'.$this->type.'_lft = single.lft, '
                .'    related.'.$this->type.'_rgt = single.rgt'
                .' WHERE single.id = related.'.$this->type.'_id '
                );
        return $cmd->execute();
    }
    
    public function moveItem($item, $parent){
            
        $childrenCount  = ($item->rgt - $item->lft - 1) / 2;
        $parentRgt      = $this->getParentRight($parent);
        
        /**
         * If we don't have a parent, we have to incrase the parentRgt manually, because we don't want this parent
         * The method getParentRight can't do this as it always returns a valid ID - we need a new one!
         */
        if($parent === NULL)
            $parentRgt+=1;
        
        /**
         * The new lft is the rgt of the left neighbor + 1 
         * The rgt of the new parent - 2 * childrenCount - 2 equals this
         */
        $newLft = $parentRgt;
        $newRgt = $newLft + 2 * $childrenCount + 1;

        /**
         * Update the nodes right from the newly inserted node
         * As we take newLft from the rgt-value of the parent, we can use it now
         */
        $movedNodeSize = $item->rgt - $item->lft + 1;
        $this->updatePosition('lft', '+', $movedNodeSize, $newLft);
        //Why -1? we want to update the direct parent too !
        $this->updatePosition('rgt', '+', $movedNodeSize, $newLft - 1);
        
        //Now, reload the node in order to get the new position
        //We need the new position, as the children need to be moved accordingly
        $class = get_class($item);
        $item = $class::model()->findByPk($item->id);
        $oldLft = $item->lft;
        $oldRgt = $item->rgt;
        
        //now overwrite with new position
        $item->lft = $newLft;
        $item->rgt = $newRgt;
        $item->straightSave();
      
        /**
         * Now move the positions of all child-nodes
         * They need to be moved by the size of the occupied area 
         * 
         */
        $moveDifference = $item->lft - $oldLft;
        $cmd = Yii::app()->db->createCommand(
                'UPDATE `'.$this->table.'` SET lft = lft + :diff, rgt = rgt + :diff WHERE'
                .' lft > :lft AND rgt < :rgt'
                );
        $cmd->bindValue(':diff', $moveDifference);
        $cmd->bindValue(':lft', $oldLft);
        $cmd->bindValue(':rgt', $oldRgt);
        
        $cmd->execute();

        /**
         * As the last step, we move everything to the opposite direction, as there's space left ^^
         * As usual, do this independently
         */

        $pos = array('lft', 'rgt');
        foreach($pos as $t){
            $cmd = Yii::app()->db->createCommand(
                    'UPDATE `'.$this->table.'` SET '.$t.' = '.$t.' - :diff WHERE'
                    .' '.$t.' >= :border'
                    );
            $cmd->bindValue(':diff', $movedNodeSize);
            $cmd->bindValue(':border', $oldLft);
            $cmd->execute();
        }
     
        // We moved a lot of things possibly over a long distance. However, the related table doesn't know. Tell him.
        $this->synchronizeDenormalization();
        
        //Flush Cache
        if(isset(Yii::app()->cache))
                Yii::app()->cache->flush();
    }
    
    public function deleteItem($item){
            
            //Remove the node itself and all of its children
            $cmd = Yii::app()->db->createCommand("DELETE FROM `".$this->table."` WHERE id=:id OR ( lft > :lft AND rgt < :rgt)");
            $cmd->bindValue(':id', $item->id);
            $cmd->bindValue(':lft', $item->lft);
            $cmd->bindValue(':rgt', $item->rgt);
            
            $cmd->execute();
            
            //Ok - now update all other records following this one
            $right = $item['rgt'];
            
            $moveSize = $item->rgt - $item->lft + 1;
            $this->updatePosition('rgt', '-', $moveSize, $right);
            $this->updatePosition('lft', '-', $moveSize, $right);
            
            //Flush Cache
            if(isset(Yii::app()->cache))
                Yii::app()->cache->flush();
            
    }
    
    public function addItem($item){
        
        $newLft = $this->getParentRight($item->parent_id);
        
        //If we don't have a root, we have to add one to the lft (because we don't take the one from the parent)
        if($item->parent_id === NULL)
            $newLft +=1;
        
        $newRgt = $newLft + 1; //Initially we don't have any children. (lft + 2*childrenCount + 1)
        
        /**
         * Increase all lft and rgt which are greater than the old parent rgt by 2*childrenCount + 2
         */
        $this->updatePosition('lft', '+', 2, $newLft);
        $this->updatePosition('rgt', '+', 2, $newLft - 1);
        
        $item->lft = $newLft;
        $item->rgt = $newRgt;
        $item->straightSave();
        
        /**
         * @todo This can be done faster... do it ^^
         */
        $this->synchronizeDenormalization();
        
        //Flush Cache
        if(isset(Yii::app()->cache))
                Yii::app()->cache->flush();
       
        return Yii::app()->db->getLastInsertID();
    }
    
    /**
     * Updates the position of the nodes having a $type greater or equal than $startRgt 
     * in direction $direction by $scalar
     * @param string $type The type (lft or rgt or both)
     * @param int $direction the direciton (+ or -)
     * @param int $scalar the amount to move by
     * @param type $startRgt the starting rgt value (note: the value is also moved) 
     */
    protected function updatePosition($type, $direction, $scalar, $startRgt){
        if(!in_array($type, array('rgt', 'lft', 'both')))
            throw new Exception('Invalid type');
            
        if(!in_array($direction, array('+', '-')))
            throw new Exception('Invalid direction');
        
        $conditions = $type.' > :startRgt';
        
        $parameters = array(':scalar' => $scalar, ':startRgt' => $startRgt);
        $types = $type != 'both' ? array($type) : array('lft', 'rgt');
            
        foreach($types as $currentType){
            $sqlType = '`'.$currentType.'`';
            $command = Yii::app()->db->createCommand(   
                'UPDATE `'.$this->table.'` SET '.$sqlType.' = '.$sqlType.$direction.':scalar WHERE '.$conditions
                );
            $command->bindValues($parameters);
            $command->execute();
        }
    }
    
    /**
     * Just a Wrapper for getParentValue
     * @uses getParentValue
     * @param int $parent the ID of the parent to use
     * @return boolean  the right position of the given parent 
     */
    protected function getParentRight($parent = NULL){
        return $this->getParentValue('rgt', $parent);
    }
    
    /**
     * Just a Wrapper for getParentValue
     * @uses getParentValue
     * @param int $parent the ID of the parent to use
     * @return boolean  the left position of the given parent 
     */
    protected function getParentLeft($parent = NULL){
        return $this->getParentValue('lft', $parent);
    }
    
    /**
     * This position gathers the given value of the given parent
     * @param string $value the value to look up
     * @param int $parent the ID of the parent. If no parent is existent, usees 0 as default value
     * @return mixed    the requested value
     */
    protected function getParentValue($value, $parent = NULL){
         //If the parent is given, we look its position up
        if($parent !== NULL){
            $cmd = Yii::app()->db->createCommand("SELECT ".$value." FROM `".$this->table."` WHERE id = :id LIMIT 1");
            $cmd->bindValue(':id', $parent);

            $val = $cmd->queryScalar();
        }
        
        //Otherwise we get the highest values possible (we add it on the right)
        else{
            $cmd = Yii::app()->db->createCommand("SELECT MAX(".$value.") FROM `".$this->table."` ");
            $val = $cmd->queryScalar();
        }
            
        //If there's no right value, we want to have 1-2 als the position of the first (root)node in the tree
        if(!$val)
            return 0;
        
        return $val;
        
    }
}
?>
