<?php

/**
 * Aro Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This class represents Access Request Objects
 * Access Request Objects try to perform specific actions on Access Control Objects
 * e.g.: Users, Processes, Machines...
 */

class Aro extends ACLObject{
    
    /**
     * This method updates the permissions o the current object in the tree
     * @param mixed $actions the actions to update for
     * @param ACLObject $obj    The ACLObject to update the permissions of(e.g.: Aro or Aco)
     * @param boolean $type The type of the update: true => grant permission, false => deny permission
     * @return boolean   
     */
    protected function updatePermissions($actions, $obj, $type){
        
        $specID = $this->existsPermissionSpec($obj);
        
        $actions = array_fill_keys($this->translateActions($actions), $type);
        
        $additional = array(
          'aro_id' => $this->id,
          'aco_id' => $obj->id
        );
        //If there's no right specification
        $key =  array(
                    'aro_lft' => $this->lft,
                    'aro_rgt' => $this->rgt,
                            
                    'aco_lft' => $obj->lft,
                    'aco_rgt' => $obj->rgt,
        );
        
        if(!$specID){
            $updates = $actions;
            
            return Yii::app()->db->createCommand()->
                    insert(self::$habtmTable,
                        array_merge($updates, $key, $additional)
                            );
        }
        
        else{
            //Build the conditions
            $conditions = "";
            $bind = array();
            foreach($key as $k => $v){
                $conditions .= $k." = :".$k." AND ";
                $bind[':'.$k] = $v;
            }
            $conditions = substr($conditions, 0, -4);
            
            //OK, just update
            return Yii::app()->db->createCommand()
                    ->update(
                            self::$habtmTable,
                            $actions,
                            $conditions,
                            $bind
                        );
        }
    }
    
    /**
     * Grants the given actions to the given object
     * @param mixed $obj any valid identifier
     * @uses ACLObject::loadObject
     * @param mixed $actions the actions to grant
     * @return type 
     */
    public function grant($obj, $actions){
        $obj = $this->loadObject($obj, new Aco());
        return $this->updatePermissions($actions, $obj, true);
    }
    
    /**
     * Denies the given actions to the given object
     * @param mixed $obj any valid identifier
     * @uses ACLObject::loadObject
     * @param mixed $actions the actions to deny
     * @return type 
     */
    public function deny($obj, $actions){
        $obj = $this->loadObject($obj, new Aco());
        return $this->updatePermissions($actions, $obj, false);
    }
    
    /**
     * Checks if there are any permission specifications for the given object
     * @param Aro $obj  the object to check for
     * @return type 
     */
    protected function existsPermissionSpec($obj){
        //Check whether there's already a connection
        $cmd = Yii::app()->db->createCommand()
                ->select('COUNT(*)')
                ->from(self::$habtmTable.' map')
                ->where(
                        array('and',
                            'map.aro_id = :aro_id', 
                            'map.aco_id = :aco_id'
                            )
                       );
        $cmd->bindValue(':aro_id', $this->id);
        $cmd->bindValue(':aco_id', $obj->id);

        $count = $cmd->queryScalar();
        
        return $count;
    } 
    
    public function init() {
        $this->tree = new Tree(self::$aroTable, 'aro');
	parent::init();	
    }
    
    public function tableName(){
        return self::$aroTable;
    }
    
    public function relations(){
        return array(
          'parent' => array(self::BELONGS_TO, 'Aro', 'parent_id'),
          'acos'   => array(
              self::MANY_MANY, 'Aro', 'tbl_aco_aro(aco_id, aro_id)'
          )   
        );
    }
    
    public function rules(){
        return array(
        );
    }
    
    /**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return User the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
    
    
}
        
?>