<?php

/**
 * RequestingActiveRecord Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This class is intended as a base for objects which try to acces other objects taking into account their rights
 * In fact this class is yet another wrapper for an AroCollection
 */
class RequestingActiveRecord extends CActiveRecord{
    
    protected $aroCollection = NULL;
    
    protected function loadAroCollection(){
        if($this->aroCollection === NULL){
            $this->aroCollection = AroCollection::model()->find('model = :model AND foreign_key = :foreign_key', 
                    array(':model' => get_class($this), 'foreign_key' => $this->id));
            
            //If there's no such Aro-Collection... use Guest ^^
            if(!$this->aroCollection){
                $this->aroCollection = AroCollection::model()->find('alias = :alias', array(':alias' => 'Guest'));
                
                //If there's no guest...
                if(!$this->aroCollection)
                    throw new RuntimeException('There is no associated AroCollection nor a guest-group');
            }
        }
    }
    
    /**
     * Looks up if the user is granted a specific action to the given object
     * @param   string|array    $obj    The object to be checked   
     * @param   string          $action the action to be performed
     * @return bool true if access is granted, false otherwise
     */
    public function may($obj, $action){
        $this->loadAroCollection();
        return $this->aroCollection->may($obj, $action);
    }
    
    /**
     * Grants the object denoted by the $obj-identifier the given actions
     * @param type $obj the object identifier
     * @param array $actions the actions to grant
     * @return bool 
     */
    public function grant($obj, $actions){
        $this->loadAroCollection();
        return $this->aroCollection->grant($obj, $actions);
    }
    
    /**
     * Denies the object denoted by the $obj-identifier the given actions
     * @param type $obj the object identifier
     * @param array $actions the actions to deny
     * @return bool 
     */
    public function deny($obj, $actions){
        $this->loadAroCollection();
        return $this->aroCollection->deny($obj, $actions);
    }
    
    /**
     * This method takes care to associate an ARO-collection with this one
     * 
     * @param CEvent $evt 
     */
    public function afterSave(){
        parent::afterSave();
        if($this->isNewRecord){
            $collection = new AroCollection();
            $collection->model = get_class($this);
            $collection->foreign_key = $this->getPrimaryKey();
            if(!$collection->save())
                throw new RuntimeError("Unable to save Aro-Collection");
        }
    }
    
    /**
     * This method takes care that every associated ACL-objects are properly removed
     */
    public function beforeDelete(){
        //Ok he has the right to do that - remove all the ACL-objects associated with this object
        $aro = AroCollection::find('model = :model AND foreign_key = :key', array(':model' => get_class($this), ':key' => $this->id));
        
        if(!$aro)
            throw new RuntimeException('No associated Aro-Collection!');
        $aro->delete();
    }
    
}
?>
