<?php

/**
 * RestrictedTreeActiveRecord Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This class is intented for restricted Active-Records which should likewise be in a tree-structure
 * with permission inheritance.
 * For example: A virtual filesystem consists of files. Each file can be moved within the filesystem
 * and this way be assigned new parents and thus new permissions.
 */

class RestrictedTreeActiveRecord extends RestrictedActiveRecord{
    
    
    /**
      * @var int the parent-ID: we track changes and if they occur, we also change the permissions
      */
    protected $oldParentID = NULL;
    
    public function afterFind() {
        parent::afterFind();
        $this->oldParentID = $this->parent_id;
    }

    public function afterSave() {
        parent::afterSave();
        //If the Owner has changed - quit old parent in permission system and move to new!
        if ($this->oldParentID != $this->parent_id) {
            //@Todo: implement that in a faster way - the tree knows how, just the abstraction layer wasn't adapted accordingly
            //@see Tree::move
            //First: find the acoCollection associated to yourself
            $aco = AcoCollection::model()->find('model = :model AND foreign_key = :id', array(':model' => get_class($this), ':id' => $this->id));

            if (!$aco)
                throw new RuntimeException('Unable to find associated Aco-collection!');

            if ($this->parent_id != NULL) {
                if (!$aco->leave(array('model' => get_class($this), 'foreign_key' => $this->parent_id)))
                    throw new RuntimeException('Unable to leave old parent-aco');
            }

            $aco2 = AcoCollection::model()->find('model = :model AND foreign_key = :parent_id', array(':model' => get_class($this), ':parent_id' => $this->parent_id));
            if (!$aco->join($aco2))
                throw new RuntimeException('Unable to choose new parent-aco');
        }
    }
}
?>
